# RC5_Recorder_3.py
# Signal von PSEUDO-RC-5-Fernbedienung empfangen (z. B. mit Keuwl-Remote-App)
# Länge der High- und Low-Phasen (in us) in einer Liste speichern, auf dem Terminal ausgeben und in einer Datei speichern (Record)
# Signal aus Datei senden (Play)

# Das Programm liefert beim Empfangen fast (bis auf wenige %) dieselben Werte wie Logic Analyzer, z. B. für Taste 0:
# Ergebnis: [740, 964, 784, 977, 1669, 981, 780, 976, 808, 955, 808, 948, 789, 972, 812, 952, 808, 951, 785, 1901, ...]
# Das bedeutet insbesondere: Die Keuwl-Remote-App arbeitet (zumindest mit meinem Handy) nicht sehr genau!
# Ergebnis mit TTGO: [729, 991, 768, 984, 1662, 982, 781, 984, 777, 990, 769, 988, 772, 985, 776, 984, 777, 973, 784, 984, 777, 984, 776, 984, 777, 7285]
# Record-Funktion getestet am 13.10.2024
# Read-Funktion für Rekorder getestet am 14.10.2024
# Prüfsumme für Liste der Zeitintervalle; sollte ungefähr gleich T_max sein
# Record und Play mit majority-Fernbedienung (Internetradio; T_max = 80_000, Datei: majority_1.txt für Ein/Ausschalten) getestet am 16.10.2024; funktioniert!


from time import ticks_us, sleep_us, ticks_diff
from machine import Pin, PWM

signal_in = Pin(25, Pin.IN) # für Empfangssignal (Aufnahme); IR-Receiver
signal_out = Pin(25, Pin.OUT) # für Empfangssignal (Wiedergabe); IR-Transmitter
pwm_led = PWM(signal_out, freq=36_000, duty_u16=0) # 0 <= duty_u16 < 2^^16=65536
T_max = 30_000 # maximale Signallänge (in us); bei RC-5 unter 30_000; bei NEC deutlich mehr (ca. 75_000)

############################## Funktionen ######################################
def wait_for_high():
    global T_Start
    while (signal_in.value() == 0) and (ticks_us() - T_Start < T_max):  # solange keine Zeitüberschreitung vorliegt
        pass
    Te = ticks_us()
    return Te

def wait_for_low():
    while signal_in.value() == 1:
        pass
    Te = ticks_us()
    return Te

def save(list_file, signal_list): # T_List (s. u.) auf ESP32 als txt-Datei speichern:
    f = open(list_file, 'w')
    for delta_T in signal_list:
        f.write(str(delta_T))
        f.write('\n')
    f.close()
    
def read(list_file): # gespeicherte Daten aus txt-Datei lesen und in Listenform zurückgeben
    T_list = []
    f = open(list_file, 'r')
    while True:
        line = f.readline()
        line = line[0:-1] # \n abschneiden
        if len(line) != 0 : 
            T_list.append(int(line))
        else:
            break
    f.close()
    return T_list

def send_symbol(b): # b ist True (=1) oder False (=0)
    global T
    if b:
        pwm_led.duty_u16(0)  
        sleep_us(T)
        pwm_led.duty_u16(32768) 
        sleep_us(T)
    else:
        pwm_led.duty_u16(32768)  
        sleep_us(T)
        pwm_led.duty_u16(0)
        sleep_us(T)


######################### Aufnahme/Speichern eines Signals (record) ##########################

def record():
    global T_Start
    T_list = []
    # Auf erste positive Flanke warten; Startzeit T_Start merken:
    while signal_in.value() == 0:
        pass
    T_Start = ticks_us() # Beginn des ersten High-Signals
    Ta = T_Start

    # Solange seit dem Start T_max nicht überschritten werden...
    while (ticks_diff(ticks_us(), T_Start)) < T_max: 
        Te = wait_for_low()  # Zeit beim Ende der (nächsten) High-Phase
        deltaT = Te - Ta # Dauer des High-Zustands
        T_list.append(deltaT)
        Ta = Te
        
        Te = wait_for_high() # Zeit beim Ende der (nächsten) Low-Phase; liefert bei Zeitüberschreitung die Restzeit bis T_max
        deltaT = Te - Ta # Dauer des Low-Zustands
        T_list.append(deltaT)
        Ta = Te

    print('Zeitintervall-Liste', T_list) # Liste der Zeitintervalle für High-Low-High-Low-... Letztes Zeitintervall ist für Low?
    # Prüfsumme sollte in etwa gleich T_max sein
    summe = 0
    for deltaT in T_list:
        summe = summe + deltaT
    print('Prüfsumme:', summe)
    print()
    fname = input('Name der Signal-Datei:')
    save(fname, T_list)
    print('Daten gespeichert')


################################### Laden und Wiedergabe eines Signals #####################################################################

def read(list_file): # gespeicherte Daten aus Flash lesen und in Listenform zurückgeben
    T_list = []
    try:
        f = open(list_file, 'r')
        while True:
            line = f.readline()
            line = line[0:-1] # \n abschneiden
            if len(line) != 0 : 
                T_list.append(int(line))
            else:
                break
        f.close()
    except:
        print("Datei", list_file, 'nicht gefunden')    
    return T_list

def play(fname):
    T_list = read(fname)
    if T_list != []:
        for i in range(len(T_list) // 2):
            # High-Signal...
            pwm_led.duty_u16(32768) 
            delta_T_high = T_list[2*i]
            sleep_us(delta_T_high)
            # Low-Signal...
            pwm_led.duty_u16(0)
            delta_T_low = T_list[2*i+1]
            sleep_us(delta_T_low)
        print('Signal gesendet')
    else:
        print('kein Signal gesendet')        
        
####################### Hauptprogramm ###################################################################

print('######## Rekorder für Pseudo-RC-5-Signale ##########')
print()
print('Geben Sie ein:')
print('r für record')
print('p für play')
print()
c = input('Eingabe:')

if c == 'r':
    print('Warte auf Fernbedienungssignal...')
    record()
elif c == 'p':
    list_file_name = input('Name der txt-Datei des Signals:')
    play(list_file_name)   
else:
    print('Auswahl ungültig')
    